/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.apache.shardingsphere.core.strategy.route.inline;

import com.google.common.collect.Lists;
import org.apache.shardingsphere.api.config.sharding.strategy.InlineShardingStrategyConfiguration;
import org.apache.shardingsphere.core.strategy.route.value.ListRouteValue;
import org.apache.shardingsphere.core.strategy.route.value.RouteValue;
import org.junit.Before;
import org.junit.Test;

import java.util.Collection;
import java.util.List;

import static org.hamcrest.CoreMatchers.is;
import static org.junit.Assert.assertThat;

public class InlineShardingStrategyTest {
    
    private InlineShardingStrategy shardingStrategy;
    
    @Before
    public void setUp() throws Exception {
        InlineShardingStrategyConfiguration shardingStrategyConfig = new InlineShardingStrategyConfiguration("order_id", "t_order_${order_id % 4}");
        shardingStrategy = new InlineShardingStrategy(shardingStrategyConfig);
    }
    
    @Test
    public void assertDoSharding() {
        List<String> availableTargetNames = Lists.newArrayList("t_order_0", "t_order_1", "t_order_2", "t_order_3");
        List<RouteValue> shardingValues = Lists.<RouteValue>newArrayList(new ListRouteValue("order_id", "t_order", Lists.newArrayList(0, 1, 2, 3)));
        Collection<String> actual = shardingStrategy.doSharding(availableTargetNames, shardingValues);
        assertThat(actual.size(), is(4));
    }
    
    @Test
    public void assertDoShardingWithNonExistNodes() {
        List<String> availableTargetNames = Lists.newArrayList("t_order_0", "t_order_1");
        List<RouteValue> shardingValues = Lists.<RouteValue>newArrayList(new ListRouteValue("order_id", "t_order", Lists.newArrayList(0, 1, 2, 3)));
        Collection<String> actual = shardingStrategy.doSharding(availableTargetNames, shardingValues);
        assertThat(actual.size(), is(2));
    }
    
    @Test
    public void assertGetShardingColumns() {
        assertThat(shardingStrategy.getShardingColumns().size(), is(1));
        assertThat(shardingStrategy.getShardingColumns().iterator().next(), is("order_id"));
    }
}
